/*
   Copyright (c) 2014 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "includes.h"

#include "svpwm.h"
#include "pi.h"
#include "vector.h"
#include "filter.h"
#include "angle.h"
#include "fp_math.h"

extern const Timer_Handle myTimer;
extern const Swi_Handle mySwi;

PLX_ANGLE_Obj_t PhiObj;
PLX_ANGLE_Handle_t PhiHandle;

PLX_SVPWM_Obj_t SvpwmObj;
PLX_SVPWM_Handle_t SvpwmHandle;

PLX_FILT_Obj_t FilterDObj;
PLX_FILT_Handle_t FilterDHandle;

PLX_FILT_Obj_t FilterQObj;
PLX_FILT_Handle_t FilterQHandle;

PLX_PI_Obj_t PiObj;
PLX_PI_Handle_t PiHandle;

struct CONTROL_VARS ControlVars;

extern const Timer_Handle TickTimer;

void InitControl(void){
	xdc_runtime_Types_FreqHz freqHz;

	Timer_getFreq(TickTimer, &freqHz);
	ControlVars.task1Freq = (Uint16)(freqHz.lo/Timer_getPeriod(TickTimer));
	InitPuVars(ControlVars.task1Freq);

	ControlVars.ePWM_Period = (uint16_t)(SYSCLK_HZ / 2 / ControlVars.task1Freq / 2);

	ControlVars.task2Freq = ControlVars.task1Freq/TASK2_PERIOD;

	PhiHandle = PLX_ANGLE_init(&PhiObj, sizeof(PhiObj));
	PLX_ANGLE_configure(PhiHandle, PuVars.frefTsScaling, 1000);

	SvpwmHandle = PLX_SVPWM_init(&SvpwmObj,sizeof(SvpwmObj));
	PLX_SVPWM_configure(SvpwmHandle, ControlVars.ePWM_Period, 2);

	FilterDHandle = PLX_FILT_init(&FilterDObj, sizeof(FilterDObj));
	PLX_FILT_configure(FilterDHandle, Calib.Kf, 0);

	FilterQHandle = PLX_FILT_init(&FilterQObj, sizeof(FilterQObj));
	PLX_FILT_configure(FilterQHandle, Calib.Kf, 0);

	PiHandle = PLX_PI_init(&PiObj, sizeof(PiObj));

	InitPWM();
	InitADC();

	StopControl();
	ResetControl();
}

void StopControl(void){
	// disable actuators
}

void StartControl(void){
	ResetControl();
	// enable actuators
}

void ResetControl(void){
	ControlVars.task1StepCtr = 0;
	ControlVars.task2StepCtr = 0;

	PLX_ANGLE_reset(PhiHandle, 0);
	PLX_FILT_reset(FilterDHandle, 0);
	PLX_FILT_reset(FilterQHandle, 0);
	PLX_PI_configure(PiHandle, Calib.Kp, Calib.Ki);
	PLX_PI_reset(PiHandle);

	ControlVars.Vac_maxPerSoftstart = 0;
}

bool_t ProcessInputs(){
	return(TRUE);

}

void ControlTask1(){
	ControlVars.task1StepCtr++;

	SET_OPROBE(ControlVars.Vdc, 0);
	SET_OPROBE(ControlVars.Vload_alpha, 0);
	SET_OPROBE(ControlVars.Vload_beta, 0);

	// calculate maximal achievable phase voltage based on DC-link voltage (max is 1/sqrt(3)
	int16_t vsmax = Q_MUL(ControlVars.Vdc, (int16_t)(1.0 / 1.73 * 32768.0), 15);
	// apply some margin
	vsmax = Q_MUL(vsmax, (int16_t)(0.95 * 32768.0), 15);
	SET_OPROBE(ControlVars.Vac_maxPerVdc, vsmax);

	// integrate voltage angle
	uint16_t angle;
	SET_OPROBE(ControlVars.Omega, 0);
	PLX_ANGLE_accumulate(PhiHandle, ControlVars.Omega, &angle);
	SET_OPROBE(ControlVars.Phi, angle);

	// calculate sine and cosine of voltage angle
	PLX_FPMATH_lookupSinAndCos(ControlVars.Phi, &ControlVars.sinOfPhi, &ControlVars.cosOfPhi);


	// voltage set-point in rotating frame
	int16_t vd, vq;
	vd = 0;
	vq = ControlVars.Vac_set;

	// transform voltage set-point to fixed frame
	int16_t va, vb;
	PLX_VECT_invParkRot(vd, vq, &va, &vb, ControlVars.sinOfPhi, ControlVars.cosOfPhi);
	SET_OPROBE(ControlVars.Vac_set_alpha, va);
	SET_OPROBE(ControlVars.Vac_set_beta, vb);

	// svpm
	int16_t svpwmLimitingGain;
	PLX_SVPWM_calcCompareValues(SvpwmHandle,
			ControlVars.Vac_set_alpha, ControlVars.Vac_set_beta, ControlVars.Vdc,
			&ControlVars.ePWM_CmpU, &ControlVars.ePWM_CmpV, &ControlVars.ePWM_CmpW, &svpwmLimitingGain);

	// transform voltage measurements into rotating frame
	int16_t vdload, vqload;
	PLX_VECT_parkRot(ControlVars.Vload_alpha, ControlVars.Vload_beta, &vdload, &vqload, ControlVars.sinOfPhi, ControlVars.cosOfPhi);

	// filter voltage measurements
	PLX_FILT_calculateOutput(FilterDHandle, vdload, &ControlVars.VloadF_d);
	PLX_FILT_calculateOutput(FilterQHandle, vqload, &ControlVars.VloadF_q);
}

extern uint16_t DSP28x_DisableInt();
extern void DSP28x_RestoreInt(uint16_t Stat0);

void ControlTask2(){
	ControlVars.task2StepCtr++;

	// softstart
	int16_t vsmaxSoftstart = ControlVars.Vac_maxPerSoftstart;
	if(vsmaxSoftstart < Calib.Vac_limit)
	{
		vsmaxSoftstart += Calib.Vac_softstartSlew;
	}
	if(vsmaxSoftstart > Calib.Vac_limit)
	{
		vsmaxSoftstart = Calib.Vac_limit;
	}
	SET_OPROBE(ControlVars.Vac_maxPerSoftstart, vsmaxSoftstart);

	// calculate load voltage amplitude
	int16_t vd2 = Q_MUL(ControlVars.VloadF_d, ControlVars.VloadF_d, Q_IV);
	int16_t vq2 = Q_MUL(ControlVars.VloadF_q, ControlVars.VloadF_q, Q_IV);
	ControlVars.VloadF = PLX_FPMATH_sqrtQ10(vd2+vq2);

	// PI for load voltage regulation
	SET_OPROBE(ControlVars.Vload_set, Calib.Vload_set);
	int16_t error = ControlVars.Vload_set - ControlVars.VloadF;
	int16_t voutDesired;
	PLX_PI_calculateOutput(PiHandle, error, &voutDesired);

	// only allow positive values
	if(voutDesired < 0)
	{
		voutDesired = 0;
	}

	// limit voltage based on soft-start and DC-link voltage
	if(voutDesired > ControlVars.Vac_maxPerSoftstart)
	{
		voutDesired = ControlVars.Vac_maxPerSoftstart;
	}
	if(voutDesired > ControlVars.Vac_maxPerVdc)
	{
		voutDesired = ControlVars.Vac_maxPerVdc;
	}

	// update integrator, taking into account output saturation
	PLX_PI_updateIntegrator(PiHandle, voutDesired);

	// export variables for control task 1
	// (need to disable interrupts in order to export consistent set)
	uint16_t stat = DSP28x_DisableInt();
	SET_OPROBE(ControlVars.Vac_set, voutDesired);
	DSP28x_RestoreInt(stat);
}
