/*
  Copyright (c) 2014 by Plexim GmbH
  All rights reserved.

  A free license is granted to anyone to use this software for any legal
  non safety-critical purpose, including commercial applications, provided
  that:
  1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
  2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

/**
 * @file pil.h
 * @brief Public interface to PIL framework.
 * @version 3.6.0
 */

#ifndef PIL_H_
#define PIL_H_

#ifdef __cplusplus
extern "C" {
#endif


#include <stdint.h>

//@{
/** Additional types and constant definitions */
#ifndef bool_t
typedef char bool_t;
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef NULL
#define NULL 0
#endif
//@}

/**
 * Structure for configuring all PIL symbols
 */
typedef struct
{
   int q;       //!< fixed-point location
   float ref;   //!< reference value
   char *unit;  //!< unit string
} pil_var;

//@{
/** Probes Functionality */
#define PROBE_FEED_THROUGH 0
#define PROBE_OVERRIDE 1

//PIL macros for Symbol and Override Probe Definition
#define PIL_SYMBOL_DEF(name, qloc, ref, unit)\
   const pil_var PIL_V_ ## name = {qloc, ref, unit}

#define PIL_OVERRIDE_PROBE(type, name)\
      type name;\
      type name ## _probeV;\
      int16_t name ## _probeF

//Probe macro definitions
#define INIT_OPROBE(probe) \
   do{\
      probe ## _probeF = PROBE_FEED_THROUGH;\
   } while (0)

#define SET_OPROBE(probe, value) \
   do{\
      probe = value; \
      if(probe ## _probeF == PROBE_OVERRIDE){\
         probe = probe ## _probeV; \
      }\
   } while (0)

//Macro for PIL_C_x definition
#define PIL_CONFIG_DEF(type, name, value)\
  const type PIL_C_ ## name = value

/**
 * Control callback identifiers.
 * These constants identify individual actions which the PIL framework
 * requests from the application.
 */
typedef enum
{
   PIL_CLBK_LEAVE_NORMAL_OPERATION_REQ, //!< Request system to enter PIL mode (i.e. disable actuators)
   PIL_CLBK_ENTER_NORMAL_OPERATION_REQ, //!< Release system from PIL mode (i.e. allow re-enabling of actuators)
   PIL_CLBK_INITIALIZE_SIMULATION,      //!< Request processed at the PIL simulation start
   PIL_CLBK_TERMINATE_SIMULATION,       //!< Request processed at the PIL simulation termination
   PIL_CLBK_STOP_TIMERS,                //!< Freeze all timers
   PIL_CLBK_START_TIMERS                //!< Release frozen timers
} PIL_CtrlCallbackReq_t;

/**
 * Control callback type definition.
 * The PIL framework calls the application using this callback pointer.
 */
typedef void(*PIL_CtrlCallbackPtr_t)(PIL_CtrlCallbackReq_t);

/**
 * Communication callback type definition.
 * The PIL framework calls the application using this callback pointer
 * to receive incoming data and to request the transmission of outgoing messages.
 */
typedef void(*PIL_CommCallbackPtr_t)();

/**
 * Initializes the PIL framework.
 * This function must be called before any other framework functions.
 */
extern void PIL_init(void);

/**
 * Sets the mandatory link parameters
 *
 * @param  aGuid     Pointer to first element of GUID field
 * @param aCommPtr   Pointer to CommCallback
 * @return None
 */
extern void PIL_setLinkParams(const unsigned char* aGuid, PIL_CommCallbackPtr_t aCommPtr);

/**
 * Sets desired node address
 *
 * @param aNode Desired Station/Node Address
 * @return None
 */
extern void PIL_setNodeAddress(unsigned char aNode);

/**
 * Sets pointer to the PIL control callback.
 *
 * @param  aCtrlPtr   Pointer to CtrlCallback
 * @return None
 */
extern void PIL_setCtrlCallback(PIL_CtrlCallbackPtr_t aCtrlPtr);

/**
 * Sets pointer to the PIL background communication callback.
 *
 * @param  aBackgroundCommPtr   Pointer to BackgroundCommCallback
 * @return None
 */
extern void PIL_setBackgroundCommCallback(PIL_CommCallbackPtr_t aBackgroundCommPtr);

/**
 * Configures External Scope Buffer
 *
 * @param  aBufPtr   Pointer to Scope Buffer
 * @param  aBufSize  Size of Scope Buffer
 * @return None
 */
extern void PIL_setScopeBuffer(uint16_t* aBufPtr, uint16_t aBufSize);

/**
 * Indicates to the framework that the system is ready for a PIL simulation
 * Invokes a Framework State change to the Ready Mode
 * This method is used as a response to an LEAVE_NORMAL_OPERATION_REQ and has no effect in PIL Mode
 *
 * @return None
 */
extern void PIL_allowPilSimulation(void);

/**
 * Indicates to the framework that the system is not ready for a PIL simulation
 * Invokes a Framework State change to Normal Operation
 * This method is used as a response to an ENTER_NORMAL_OPERATION_REQ and has no effect in PIL Mode
 *
 * @return None
 */
extern void PIL_inhibitPilSimulation(void);

/**
 * Used to Request the PIL_CLBK_ENTER_NORMAL_OPERATION_REQ Callback from the Application
 * Only has an effect outside PIL Mode
 *
 * @return None
 */
void PIL_requestNormalMode(void);

/**
 * Used to Request the PIL_CLBK_LEAVE_NORMAL_OPERATION_REQ Callback from the Application
 * Only has an effect outside PIL Mode
 *
 * @return None
 */
void PIL_requestReadyMode(void);

/**
 * Used to check if a PIL simulation is active
 *
 * @return None
 */
extern bool_t PIL_simulationActive(void);

/**
 * Maximal length for PIL message
 */
#define PIL_MSG_DATA_LEN 40

/**
 * Receive function.
 * This function is used by the communication callback to
 * pass received characters (e.g. from the SCI link) to the PIL
 * message parser.
 *
 * @param aChar character received over communication link
 */
extern void PIL_RA_serialIn(int16_t aChar);

/**
 * Transmit function.
 * This function is used by the communication callback to
 * obtain characters from the PIL framework that are ready for
 * transmission (e.g. over the SCI link).
 *
 * @param aChar pointer to hold potential character to be transmitted
 * @return true if a character is ready for transmission and written to aChar
 */
extern bool_t PIL_RA_serialOut(int16_t* aChar);

/**
 * Receive function for Direct Link.
 * This function is used by the communication callback to store incoming messages to the PIL framework.
 *
 * IMPORTANT: This function must be called by the background task.
 *
 * This function copys the content of the buffer aBuf from the first element to the (aLen-1)th element
 * into the framework buffer where it is parsed and processed.
 *
 * @param *aBuf Pointer to the buffer where the incoming message is stored
 * @param aLen  Length of incoming message in aBuf
 * @return None
 */
extern void PIL_RA_parallelIn(uint16_t *aBuf, uint16_t aLen);

/**
 * Transmit function for Direct Link.
 * This function is used by the communication callback to
 * obtain messages(stored in an internal buffer) from the PIL framework that
 * are ready for transmission.
 *
 * IMPORTANT: This function must be called by the background task.
 *
 * If an outgoing message exists, this functions writes the whole message into the buffer specified
 * in the parameters. Furthermore, the message length is provided for further processing in the
 * communication Callback.
 *
 * @param *aBuf Pointer to the buffer where the outgoing message needs to be stored
 * @param *aLen Pointer to the variable where the length of the actually outgoing message is stored
 * @return None
 */
extern void PIL_RA_parallelOut(uint16_t *aBuf, uint16_t *aLen);

/**
 * PIL service function to be called at the beginning of the control
 * interrupt routine.
 */
extern void PIL_beginInterruptCall(void);

/**
 * PIL service function to be called in the background task. *
 *
 */
extern void PIL_backgroundCall(void);

/**
 * Scope "sample" call.
 * This function must be called periodically at the desired sample
 * rate to buffer data for the PLECS-Console scope.
 */
extern void PIL_SCOPE_sample(void);

/**
 * Scope "trigger" call.
 * This function can be used to trigger the PLECS-Console scope from software
 * (as opposed to from the GUI).
 */
extern void PIL_SCOPE_trigger(void);

#ifdef __cplusplus
}
#endif
#endif /* PIL_H_ */
